#!/bin/bash

cd ..

## set up virtual environment
python3 -m venv venv
source venv/bin/activate
pip install wheel
pip install -r requirements.txt
python -c "import classla; classla.download('sl_ssj_jos')" <<< $'Y\nresources/classla'
deactivate

## get needed repositories
git clone https://github.com/clarinsi/Obeliks4J
git clone git@gitea.cjvt.si:redmine_projects/nova_slovnica.git
git clone git@gitea.cjvt.si:ozbolt/luscenje_struktur.git
git clone git@gitea.cjvt.si:generic/data_admin.git
git clone git@gitea.cjvt.si:generic/xml_schemas.git

## set up obeliks
cd Obeliks4J
javac -encoding UTF-8 src/main/java/org/obeliks/*.java -d target/classes
cp src/main/resources/* target/classes/org/obeliks/
jar -cef org.obeliks.Tokenizer obeliks.jar -C target/classes org
cd ..

## put needed scripts in place 
cd scripts
ln -s ../nova_slovnica/python/scripts/conllu_to_xml.py .
ln -s ../nova_slovnica/python/scripts/assign_structures.py .
ln -s ../nova_slovnica/python/scripts/create_structures.py .
ln -s ../nova_slovnica/python/scripts/tei_to_dictionary.py .
ln -s ../luscenje_struktur/wani.py .
cd ..

## put needed resources in place
cd resources
mv ../Obeliks4J/obeliks.jar .
ln -s ../nova_slovnica/resources/dict.xml .
ln -s ../data_admin/resources/structures.xml .
ln -s ../data_admin/resources/structures.xsd .
ln -s ../xml_schemas/resources/schema/inventory.xsd .
ln -s ../xml_schemas/resources/schema/monolingual_dictionaries.xsd .
cd ..
